//  Copyright (c) 2000 Jan Christian Kaldestad & Hallstein Lohne
//	    USAGE: checkService <ip> <port> <showall/shownone> <services> ...
//      REQUIRED "../etc/ntpasswd" this file contain password!



#include "util.h"
#include "../../../Common/queries.h"
#include "../../../Common/kvbuffer.h"




int showUsage()
{
  cout << "  Check NT Service by Jan Christian Kaldestad and Hallstein Lohne\nUsage:\n"
    << "CheckService <ip> <port> <showall/showfail> <Servicename1> [Servicename2]\n"
    << "         ... [\"Service name3\"] [Servicename4] ..." <<endl<<endl
    << "Ex: check_ntservice hostname 1245 showfail \"NetSaint Service\" Messenger \"Fax Service\" "
    << "\nReply from NTserver(WARNING): \"NotActive:Messenger NotActive:Fax Service\""
    << "\nThis will display only failed services on destination NTserver."
    << "\n    showall - services which do not fail will also be shown.\n"
    << "    showfail - only services which are not running will be shown.\n"
    << "(NetSaint can only display about 125 characters, so be sure what to monitor.)\n"
    << "Win32 returncodes which checkService monitors:\n"
    << "SERVICE RESPOND         - NETSAINT RESPOND\n"
    << "Running                 - Returns OK (0)\n"
    << "Does not exit           - Returns WARNING (1)\n"
    << "Is paused               - Returns WARNING (1)\n"
    << "Has not been started    - Returns WARNING (1)\n"
    << "Timout on request       - Returns CRITICAL (2)\n"
    << "Check error.            - Returns CRITICAL (2)\n"
    << endl;
  return STATE_UNKNOWN;
}



bool showWorking = false;

int CheckCmdLine(int argc, char **argv, KVBuffer& sbuf)
{
  int numberOfServices = 0;
  char **lplpRecList  = new char*[4];
  char **lplpRetCodes = new char*[4];
  char	     ** lplpServices;
  int             max_sname_length = 100;


  if (argc < 5)
  {
    return showUsage();
  }

  numberOfServices = argc - 4;

  //initialize the table
  lplpServices = new char*[numberOfServices];
  lplpRecList  = new char*[numberOfServices];
  lplpRetCodes = new char*[numberOfServices];

  //copying from argv into lplpServices
  int i;
  for (i=0; i<numberOfServices; i++)
  {
    lplpServices[i] = new char [max_sname_length];
    lplpServices[i] = argv[i+4];
  }

  if ( strcmp("showall", argv[3]) == 0)
  {
    showWorking = true;
  }
  else
  {
    if ( strcmp("showfail", argv[3]) == 0)
    {
      showWorking = false;
    }
    else
    {
      return showUsage();
    }
  }

    sbuf.addKeyIntValue("id", SERVICECHECK_QUERY);

    //insert from our command-string
    char str[36];
    for (i=0; i<numberOfServices; i++)
    {
      sprintf(str, "service%d", i);
      sbuf.addKeyValue(str, lplpServices[i]);
    }

    return STATE_OK;
}


bool GetStringAndState(int argc, char** argv, KVBuffer& sbuf, KVBuffer& rbuf, int& state, char* output)
{
  int toReturn = STATE_OK;
  int i = 0;
  bool end = false;
  char str[50];
  int g = 0;
  while(!end)
  {
    sprintf(str, "service%d", i);
    const char* servName = sbuf.getValue(str);
    if (servName == 0)
    {
      end = true;
      break;
    }
    i++;

    int status = rbuf.getIntValue(servName);

    debug("Service: %d: %s : %d", i, servName, status);

    switch(status)
    {
    case SERV_SERVICE_DOES_NOT_EXIST:
    {
      g += sprintf(output + g, "NotExist:'%s' ", servName );
      if (toReturn < STATE_WARNING)
      {
        toReturn = STATE_WARNING;
      }
    }
    break;

    case SERV_SERVICE_NOT_ACTIVE:
    {
      g += sprintf(output + g, "NotActive:'%s' ", servName );
      if (toReturn < STATE_WARNING)
      {
        toReturn = STATE_WARNING;
      }
    }
    break;

    case SERV_SERVICE_REQUEST_TIMEOUT:
    {
      g += sprintf(output + g, "Timeout:'%s' ", servName );
      if (toReturn < STATE_CRITICAL)
      {
        toReturn = STATE_CRITICAL;
      }
    }
    break;

    case SERV_CHECK_ERROR:
    {
      g += sprintf(output + g, "CheckError:'%s' ", servName );
      if (toReturn < STATE_CRITICAL)
      {
        toReturn = STATE_CRITICAL;
      }
    }
    break;

    case SERV_PAUSED:
    {
      g += sprintf(output + g, "Paused:'%s' ", servName );
      if (toReturn < STATE_WARNING)
      {
        toReturn = STATE_WARNING;
      }
    }
    break;

    case SERV_RUNNING:
    {
      if (showWorking)
      {
        g += sprintf(output + g, "OK:'%s' ", servName );
      }
    }
    break;

    default:
    {
      debug("ERROR : Wrong service return status");
    }
    break;

    }; // end switch

  }; // end while


  //prints the one-and-only output
  if (!showWorking)
  {
    if (strlen(output) < 3)
    {
        cout << "All OK";
    }
    cout << output << endl;
  }

  state = toReturn;
  return true;
}
